@@ -32,8 +32,6 @@
 #include <glib/gprintf.h>
 #include <libsoup/soup-misc.h>
 #include "e-ews-item.h"
-#include "e-ews-connection.h"
-#include "e-ews-message.h"
 
 #ifdef G_OS_WIN32
 
@@ -179,6 +177,7 @@ struct _EEwsItemPrivate {
 
 	gchar *uid;
 	gchar *timezone;
+	gchar *contact_photo_id;
 
 	GSList *to_recipients;
 	GSList *cc_recipients;
@@ -188,7 +187,7 @@ struct _EEwsItemPrivate {
 	EwsMailbox *sender;
 
 	GSList *modified_occurrences;
-	GSList *attachments_list;
+	GSList *attachments_ids;
 	GSList *attendees;
 
 	EwsId *calendar_item_accept_id;
@@ -255,42 +254,26 @@ e_ews_item_dispose (GObject *object)
 	g_free (priv->timezone);
 	priv->timezone = NULL;
 
-	if (priv->to_recipients) {
-		g_slist_foreach (priv->to_recipients, (GFunc) e_ews_mailbox_free, NULL);
-		g_slist_free (priv->to_recipients);
-		priv->to_recipients = NULL;
-	}
+	g_free (priv->contact_photo_id);
+	priv->contact_photo_id = NULL;
 
-	if (priv->cc_recipients) {
-		g_slist_foreach (priv->cc_recipients, (GFunc) e_ews_mailbox_free, NULL);
-		g_slist_free (priv->cc_recipients);
-		priv->cc_recipients = NULL;
-	}
+	g_slist_free_full (priv->to_recipients, (GDestroyNotify) e_ews_mailbox_free);
+	priv->to_recipients = NULL;
 
-	if (priv->bcc_recipients) {
-		g_slist_foreach (priv->bcc_recipients, (GFunc) e_ews_mailbox_free, NULL);
-		g_slist_free (priv->bcc_recipients);
-		priv->bcc_recipients = NULL;
-	}
+	g_slist_free_full (priv->cc_recipients, (GDestroyNotify) e_ews_mailbox_free);
+	priv->cc_recipients = NULL;
 
-	if (priv->modified_occurrences) {
-		g_slist_foreach (priv->modified_occurrences, (GFunc) g_free, NULL);
-		g_slist_free (priv->modified_occurrences);
-		priv->modified_occurrences = NULL;
-	}
+	g_slist_free_full (priv->bcc_recipients, (GDestroyNotify) e_ews_mailbox_free);
+	priv->bcc_recipients = NULL;
 
-	if (priv->attachments_list) {
-		g_slist_foreach (priv->attachments_list, (GFunc) g_free, NULL);
-		g_slist_free (priv->attachments_list);
-		priv->attachments_list = NULL;
-	}
+	g_slist_free_full (priv->modified_occurrences, g_free);
+	priv->modified_occurrences = NULL;
 
-	if (priv->attendees) {
-		g_slist_foreach (priv->attendees, (GFunc) ews_item_free_attendee, NULL);
-		g_slist_free (priv->attendees);
-		priv->attendees = NULL;
+	g_slist_free_full (priv->attachments_ids, g_free);
+	priv->attachments_ids = NULL;
 
-	}
+	g_slist_free_full (priv->attendees, (GDestroyNotify) ews_item_free_attendee);
+	priv->attendees = NULL;
 
 	if (priv->calendar_item_accept_id) {
 		g_free (priv->calendar_item_accept_id->id);
@@ -535,11 +518,8 @@ parse_categories (EEwsItemPrivate *priv,
 	ESoapParameter *subparam;
 
 	/* release all the old data (if any) */
-	if (priv->categories) {
-		g_slist_foreach (priv->categories, (GFunc) g_free, NULL);
-		g_slist_free (priv->categories);
-		priv->categories = NULL;
-	}
+	g_slist_free_full (priv->categories, g_free);
+	priv->categories = NULL;
 
 	/* categories are an array of <string> */
 	for (subparam = e_soap_parameter_get_first_child (param);
@@ -592,16 +572,29 @@ process_attachments_list (EEwsItemPrivate *priv,
 {
 	ESoapParameter *subparam, *subparam1;
 
-	GSList *list = NULL;
+	GSList *ids = NULL;
 
 	for (subparam = e_soap_parameter_get_first_child (param); subparam != NULL; subparam = e_soap_parameter_get_next_child (subparam)) {
+		gchar *id;
 
 		subparam1 = e_soap_parameter_get_first_child_by_name (subparam, "AttachmentId");
+		id = e_soap_parameter_get_property (subparam1, "Id");
 
-		list = g_slist_append (list, e_soap_parameter_get_property (subparam1, "Id"));
+		subparam1 = e_soap_parameter_get_first_child_by_name (subparam, "IsContactPhoto");
+		if (subparam1) {
+			gchar *value = e_soap_parameter_get_string_value (subparam1);
+			if (g_strcmp0 (value, "true") == 0) {
+				priv->contact_photo_id = id;
+				g_free (value);
+				continue;
+			}
+			g_free (value);
+		}
+
+		ids = g_slist_append (ids, id);
 	}
 
-	priv->attachments_list = list;
+	priv->attachments_ids = ids;
 	return;
 }
 
@@ -1452,6 +1445,14 @@ e_ews_item_mailbox_from_soap_param (ESoapParameter *param)
 	if (subparam)
 		mb->mailbox_type = e_soap_parameter_get_string_value (subparam);
 
+	subparam = e_soap_parameter_get_first_child_by_name (param, "ItemId");
+	if (subparam) {
+		EwsId *id = g_new0 (EwsId, 1);
+		id->id = e_soap_parameter_get_property (subparam, "Id");
+		id->change_key = e_soap_parameter_get_property (subparam, "ChangeKey");
+		mb->item_id = id;
+	}
+
 	if (!mb->email && !mb->name) {
 		e_ews_mailbox_free (mb);
 		mb = NULL;
@@ -1493,7 +1494,7 @@ e_ews_item_get_attachments_ids (EEwsItem *item)
 {
 	g_return_val_if_fail (E_IS_EWS_ITEM (item), NULL);
 
-	return item->priv->attachments_list;
+	return item->priv->attachments_ids;
 }
 
 gchar *
@@ -1517,28 +1518,28 @@ e_ews_embed_attachment_id_in_uri (const gchar *olduri,
 		g_warning ("Failed to move attachment cache file [%s -> %s]: %s\n", tmpfilename, filename, strerror (errno));
 	}
 
+	g_free (tmpfilename);
 	g_free (tmpdir);
 
 	return g_filename_to_uri (filename, NULL, NULL);
 }
 
-gchar *
+EEwsAttachmentInfo *
 e_ews_dump_file_attachment_from_soap_parameter (ESoapParameter *param,
                                                 const gchar *cache,
-                                                const gchar *comp_uid,
-                                                gchar **attach_id)
+                                                const gchar *comp_uid)
 {
 	ESoapParameter *subparam;
-	const gchar *param_name;
+	const gchar *param_name, *tmpfilename;
 	gchar *name = NULL, *value, *filename, *dirname;
 	guchar *content = NULL;
 	gsize data_len = 0;
-	gchar *tmpdir, *tmpfilename;
+	gchar *tmpdir;
+	EEwsAttachmentInfo *info;
 
 	g_return_val_if_fail (param != NULL, NULL);
 
 	/* Parse element, look for filename and content */
-	*attach_id = NULL;
 	for (subparam = e_soap_parameter_get_first_child (param); subparam != NULL; subparam = e_soap_parameter_get_next_child (subparam)) {
 		param_name = e_soap_parameter_get_name (subparam);
 
@@ -1550,49 +1551,56 @@ e_ews_dump_file_attachment_from_soap_parameter (ESoapParameter *param,
 			value = e_soap_parameter_get_string_value (subparam);
 			content = g_base64_decode (value, &data_len);
 			g_free (value);
-		} else if (g_ascii_strcasecmp (param_name, "AttachmentId") == 0) {
-			*attach_id = e_soap_parameter_get_property (subparam, "Id");
 		}
 	}
 
 	/* Make sure we have needed data */
-	if (!content || !name || !*attach_id) {
+	if (!content || !name) {
 		g_free (name);
 		g_free (content);
-		g_free (*attach_id);
 		return NULL;
 	}
 
-	tmpfilename = (gchar *) content;
-	tmpdir = g_strndup (tmpfilename, g_strrstr (tmpfilename, "/") - tmpfilename);
+	if (cache) {
+		info = e_ews_attachment_info_new (E_EWS_ATTACHMENT_INFO_TYPE_URI);
 
-	dirname = g_build_filename (tmpdir, comp_uid, NULL);
-	if (g_mkdir_with_parents (dirname, 0775) == -1) {
-		g_warning ("Failed create directory to place file in [%s]: %s\n", dirname, strerror (errno));
+		tmpfilename = (gchar *) content;
+		tmpdir = g_strndup (tmpfilename, g_strrstr (tmpfilename, "/") - tmpfilename);
+
+		dirname = g_build_filename (tmpdir, comp_uid, NULL);
+		if (g_mkdir_with_parents (dirname, 0775) == -1) {
+			g_warning ("Failed create directory to place file in [%s]: %s\n", dirname, strerror (errno));
+		}
+
+		filename = g_build_filename (dirname, name, NULL);
+		if (g_rename (tmpfilename, filename) != 0) {
+			g_warning ("Failed to move attachment cache file [%s -> %s]: %s\n",
+					tmpfilename, filename, strerror (errno));
+		}
+
+		g_free (dirname);
+		g_free (tmpdir);
+		g_free (name);
+		g_free (content);
+
+		/* Return URI to saved file */
+		e_ews_attachment_info_set_uri (info, g_filename_to_uri (filename, NULL, NULL));
+		g_free (filename);
+	} else {
+		info = e_ews_attachment_info_new (E_EWS_ATTACHMENT_INFO_TYPE_INLINED);
+		e_ews_attachment_info_set_inlined_data (info, content, data_len);
 	}
-
-	filename = g_build_filename (dirname, name, NULL);
-	if (g_rename (tmpfilename, filename) != 0) {
-		g_warning ("Failed to move attachment cache file [%s -> %s]: %s\n", tmpfilename, filename, strerror (errno));
-	}
-
-	g_free (dirname);
-	g_free (tmpdir);
-	g_free (name);
-	g_free (content);
-
-	/* Return URI to saved file */
-	name = g_filename_to_uri (filename, NULL, NULL);
-	g_free (filename);
-	return name;
+	return info;
 }
 
-gchar *
+EEwsAttachmentInfo *
 e_ews_item_dump_mime_content (EEwsItem *item,
                               const gchar *cache)
 {
+	EEwsAttachmentInfo *info = e_ews_attachment_info_new (E_EWS_ATTACHMENT_INFO_TYPE_URI);
 	gchar *filename, *surename, *dirname;
-	gchar *tmpdir, *tmpfilename;
+	gchar *tmpdir, *uri;
+	const gchar *tmpfilename;
 
 	g_return_val_if_fail (item->priv->mime_content != NULL, NULL);
 
@@ -1610,14 +1618,17 @@ e_ews_item_dump_mime_content (EEwsItem *item,
 		g_warning ("Failed to move attachment cache file");
 	}
 
+	uri = g_filename_to_uri (filename, NULL, NULL);
+	e_ews_attachment_info_set_uri (info, uri);
+
+	g_free (uri);
 	g_free (filename);
 	g_free (dirname);
 	g_free (tmpdir);
-	g_free (tmpfilename);
 	g_free (surename);
 
 	/* Return URI to saved file */
-	return g_filename_to_uri (filename, NULL, NULL);
+	return info;
 }
 
 const GSList *
@@ -1982,6 +1993,14 @@ e_ews_item_get_tzid (EEwsItem *item)
 	return item->priv->timezone;
 }
 
+const gchar *
+e_ews_item_get_contact_photo_id (EEwsItem *item)
+{
+	g_return_val_if_fail (E_IS_EWS_ITEM (item), NULL);
+
+	return item->priv->contact_photo_id;
+}
+
 EwsResolveContact *
 e_ews_item_resolve_contact_from_soap_param (ESoapParameter *param)
 {
